Skip to contents

Tukey's studentized range distribution, used for Tukey's honestly significant differences test in ANOVA.

Usage

Tukey(nmeans = numeric(), df = numeric(), nranges = numeric())

Arguments

nmeans

Sample size for each range.

df

Degrees of freedom.

nranges

Number of groups being compared.

Value

A Tukey object.

Details

We recommend reading this documentation on https://zeileis.github.io/distributions3/, where the math will render with additional detail and much greater clarity.

Support: \(R^+\), the set of positive real numbers.

Other properties of Tukey's Studentized Range Distribution are omitted, largely because the distribution is not fun to work with.

Examples


set.seed(27)

X <- Tukey(4L, 16L, 2L)
X
#> [1] "Tukey(nmeans = 4, df = 16, nranges = 2)"

cdf(X, 4)
#> [1] 0.9009192
quantile(X, 0.7)
#> [1] 3.075961