Tools to create and manipulate probability distributions using S3. Generics random(), pdf(), cdf() and quantile() provide replacements for base R's r/d/p/q style functions. Functions and arguments have been named carefully to minimize confusion for students in intro stats courses. The documentation for each distribution contains detailed mathematical notes.
Maintainer: Alex Hayes alexpghayes@gmail.com (ORCID)
Authors:
Ralph Moller-Trane
Daniel Jordan dandermotj@gmail.com
Paul Northrop p.northrop@ucl.ac.uk
Moritz N. Lang moritz.n.lang@gmail.com (ORCID)
Achim Zeileis Achim.Zeileis@R-project.org (ORCID)
Other contributors:
Emil Hvitfeldt emilhhvitfeldt@gmail.com (ORCID) [contributor]
Bruna Wundervald brunadaviesw@gmail.com [contributor]
Alessandro Gasparini alessandro.gasparini@ki.se [contributor]