powered by
Approximates an empirical distribution with a theoretical one
fit_mle(d, x, ...)
A distribution (the same kind as d) where the parameters are the MLE estimates based on x.
d
x
A probability distribution object such as those created by a call to Bernoulli(), Beta(), or Binomial().
Bernoulli()
Beta()
Binomial()
A vector of data to compute the likelihood.
Unused. Unevaluated arguments will generate a warning to catch mispellings or other possible errors.
X <- Normal() fit_mle(X, c(-1, 0, 0, 0, 3))
Run the code above in your browser using DataLab