Learn R Programming

distributions3 (version 0.3.0)

hasS3method: Check available support for S3 method

Description

Evaluates whether or not there is support for a given S3 method for specific objects.

Usage

hasS3method(method, classes)

Value

Returns TRUE if the method exists for one of the given classes, else FALSE.

Arguments

method

character, name of the method (e.g., "is_continuous", "print", ...)

classes

character vector of length > 0, classes to check.