pdf.Cauchy: Evaluate the probability mass function of a Cauchy distribution
Description
Evaluate the probability mass function of a Cauchy distribution
Usage
# S3 method for Cauchy
pdf(d, x, drop = TRUE, ...)
# S3 method for Cauchy
log_pdf(d, x, drop = TRUE, ...)
Value
In case of a single distribution object, either a numeric
vector of length probs (if drop = TRUE, default) or a matrix with
length(x) columns (if drop = FALSE). In case of a vectorized distribution
object, a matrix with length(x) columns containing all possible combinations.