pdf.Weibull: Evaluate the probability mass function of a Weibull distribution
Description
Please see the documentation of Weibull() for some properties
of the Weibull distribution, as well as extensive examples
showing to how calculate p-values and confidence intervals.
Usage
# S3 method for Weibull
pdf(d, x, drop = TRUE, ...)
# S3 method for Weibull
log_pdf(d, x, drop = TRUE, ...)
Value
In case of a single distribution object, either a numeric
vector of length probs (if drop = TRUE, default) or a matrix with
length(x) columns (if drop = FALSE). In case of a vectorized distribution
object, a matrix with length(x) columns containing all possible combinations.