Compute the sufficient statistics for the Binomial distribution from data
# S3 method for Binomial
suff_stat(d, x, ...)A Binomial object.
A vector of zeroes and ones.
Unused.
A named list of the sufficient statistics of the Binomial distribution:
successes: The total number of successful trials.
experiments: The number of experiments run.
trials: The number of trials run per experiment.