support.Erlang: Return the support of the Erlang distribution
Description
Return the support of the Erlang distribution
Usage
# S3 method for Erlang
support(d)
Arguments
d
An Erlang object created by a call to Erlang().
Value
A vector of length 2 with the minimum and maximum value of the support.