support.Weibull: Return the support of the Weibull distribution
Description
Return the support of the Weibull distribution
Usage
# S3 method for Weibull
support(d)
Arguments
d
An Weibull object created by a call to Weibull().
Value
A vector of length 2 with the minimum and maximum value of the support.