Learn R Programming

distributions3 (version 0.3.0)

support.Empirical: Return the support of the Empirical distribution

Description

Though the support of an empirical distribution is defined by its unique numeric values this function returns only the range, i.e., the lowest (minimum) and highest (maximum) observation as the outer bounds of the support.

Usage

# S3 method for Empirical
support(d, drop = TRUE, ...)

Value

In case of a single distribution object, a numeric vector of length 2 with the minimum and maximum value of the support (if drop = TRUE, default) or a matrix with 2 columns. In case of a vectorized distribution object, a matrix with 2 columns containing all minima and maxima.

Arguments

d

an Empirical object created by a call to Empirical().

drop

logical. Should the result be simplified to a vector if possible?

...

currently not used.

See Also

Other Empirical distribution: Empirical(), cdf.Empirical(), dempirical(), pdf.Empirical(), quantile.Empirical(), random.Empirical()